//
// Simple Mesh Class
//
// 
#ifndef Mesh_h
#define Mesh_h

// Helper Structures
struct TriIntersect 
{
	int Index;
	float deltaEdge1, deltaEdge2;
};

// Mesh Class
class CMesh
{
public:
	enum eSelectionMode{ SELECT_ADD, SELECT_SUB };
	enum eTriFlags{ TF_SELECTED = 1, TF_BACKFACING = (1<<1) };	

	// Functions
	CMesh();
	~CMesh();
	bool Load( const char *sFilename );
	bool DataIO( class CDataTransfer& IO );
	void CalculateFaceNormals( );
	void CalculateVertNormals( );
	void ClearTriFlags( );

	void Render( class CRenderer *pRender, bool bWireframe );
	void RenderNormals( class CRenderer *pRender, float NormalLength );
	void RenderVertices( class CRenderer *pRender );
	void RenderUVs( CRenderer *pRender );
	
	void ClearSelection();
	void SetSelectionMode( eSelectionMode nMode );
	void MarkBackfacing( CVec3 CamPos );
	
	bool LineSelect( const CVec3 &LP1, const CVec3 &LP2 );
	TriIntersect LineIntersection( const CVec3 &LP1, const CVec3 &LP2 );
	bool GetTexCoordsFromLine( const CVec3 &LP1, const CVec3 &LP2, CVec2& TexCoords );
	int FrustumSelect( CVec3 Normals[4], CVec3 Points[8] );

private:
	void Allocate( int nVerts, int nTris );
	void Deallocate( );

	CVec2 GetTexCoords( const TriIntersect& Tri );
	void SelectTriangle( int nTri );
	
	// Data		
	int m_nbVerts;
	int m_nbTris;

	CVec3 *m_pVerts;
	CVec3 *m_pVertNormals;
	CVec3 *m_pTriNormals;
	CVec2 *m_pUVs;
	unsigned int *m_pTris;
	int *m_pTriFlags;

	CVec3 m_HitP;

	eSelectionMode m_SelectMode;
};

#endif