//
// A simple paint class
// by Jonathan Kreuzer
//
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <math.h>
#include "vec3.h"
#include "texture.h"
#include "mesh.h"
#include "renderInterface.h"
#include "paint.h"

//
//
//
CTexPaint::CTexPaint()
{
	m_BrushSize = 2.0f;
	m_BrushColor.White();

	m_TexHeight = 256.0f;
	m_TexWidth = 256.0f;
}

//
// Brush Properties
//
void CTexPaint::SetBrushColor( SColor Color )
{
	m_BrushColor = Color;
}

SColor CTexPaint::GetBrushColor()
{
	return m_BrushColor;
}

void CTexPaint::IncreaseBrushSize()
{
	m_BrushSize += 1.0f;
	if ( m_BrushSize > 20.0f ) {
		m_BrushSize = 20.0f;
	}
}

void CTexPaint::DecreaseBrushSize()
{
	m_BrushSize -= 1.0f;
	if ( m_BrushSize < 1.0f ) {
		m_BrushSize = 1.0f;
	}
}

//
// Painting with the Brush
//
void CTexPaint::BrushDown( CVec2 TargetPos, CRenderer* pRender )
{
	m_bPaint = true;
	m_BrushPos = TargetPos;

	pRender->UpdateTexture( m_BrushPos, m_BrushColor, m_BrushSize );
}

//
// Draw a line from current BrushPos to TargetPos
//
void CTexPaint::DrawLine( CVec2 TargetPos, CRenderer* pRender )
{
	int nbSteps = 0;
	CVec2 Delta( TargetPos - m_BrushPos );

	if ( fabsf(Delta.x) > fabsf(Delta.y) ) 
		nbSteps = (int)ceil(fabsf(Delta.x) * m_TexWidth );
	else
		nbSteps = (int)ceil(fabsf(Delta.y) * m_TexHeight ); 

	if ( nbSteps == 0 ) {
		nbSteps = 1;
	} else {
		Delta /= (float)nbSteps;
	}

	for ( int i = 0; i < nbSteps; i++ )
	{
		m_BrushPos += Delta;
		pRender->UpdateTexture( m_BrushPos, m_BrushColor, m_BrushSize );
	}
}

//
// When the brush moves, draw on the texture
//
void CTexPaint::BrushMove( CVec2 Pos, CRenderer* pRender )
{
	if ( m_bPaint ) 
	{
		float Distance = ( Pos - m_BrushPos ).Magnitude();

		// Temporary Cheap Hack : Don't draw line if distance is too long, since the brush probably skipped
		// to a different area for the 3d Texture.
		if ( Distance > (10.0f / m_TexWidth) ) {
			pRender->UpdateTexture( Pos, m_BrushColor, m_BrushSize );
		} else {
			DrawLine( Pos, pRender );
		}
	}
	m_BrushPos = Pos;
}

void CTexPaint::BrushUp()
{
	m_bPaint = false;
}

