#ifndef CRenderer_h
#define CRenderer_h

class CVec3;
class CMatrix;
struct SColor;

class CRenderer
{
public:
	CRenderer();
	int InitRenderWindow( HWND hWnd, int bStencil, int bDoubleBuffer );
	void Shutdown( );
	void SetInitialState( );
	int IsInitialized();
	void BufferSwap( );
	void ClearBuffer( );
	bool CheckError( );
	
	void SetViewportSize( int nWidth, int nHeight );
	void GetViewportSize( int &nWidth, int &nHeight );
	void SetProjection3D( );
	void SetProjection2D( );
	void SetProjection2D_0_1( );
	
	void SetColor( float r, float g, float b );
	void SetLineMode( bool bActive );
	void SetLighting( bool bActive );
	void SetTexturing( bool bActive );
	void SetMatrix( const CMatrix &M );
	void SetPointSize( float Size );

	void DrawMesh( CVec3 *pVertices, unsigned int *pTris, int nbTris );
	void DrawMeshTextured( CVec3 *pVertices, CVec3 *pNormals, unsigned int *pTris, float *pUVs, int nbTris );
	void DrawLine( const CVec3 &P1, const CVec3 &P2 );
	void DrawTri2D( const CVec2 &P1, const CVec2 &P2, const CVec2 &P3 );
	void DrawTexturedRect( float StartX, float StartY, float Width, float Height );
	void HighlightTri( const CVec3 &P1, const CVec3 &P2, const CVec3 &P3 );
	void HighlightVert( const CVec3 &Vert );
	void DrawPoint( const CVec3& P );
	void WireRect( float x1, float y1, float x2, float y2 );
	void DrawSelectionBox( int X1, int Y1, int X2, int Y2 );

	// Textures
	void NewTexture( int Width, int Height, unsigned char* pImage );
	void ActivateTexture( );
	void UpdateTexture( const CVec2& Coord, const SColor& Color, float Size );
	
private:
	void PushMatrices();
	void PopMatrices();

	bool m_bInitialized;
	HDC m_hDC;
	HGLRC m_hGLRC;
	HWND m_hWnd;
	int m_nWidth, m_nHeight;

	// State vars
	bool m_bLighting;
	bool m_bTextured;

	// Textures
	int m_nbTextures;
	unsigned int m_BindTexId;
};

#endif