#ifndef CVec3_h
#define CVec3_h

class CVec3
{
public:
	// Data
	float x, y, z;

	// Ctors
	CVec3( float InX, float InY, float InZ ) : x(InX), y(InY), z(InZ)
		{
		}
	CVec3( ) : x(0), y(0), z(0)
		{
		}
	// Setting
	void SetZero()
		{
		x = 0;
		y = 0;
		z = 0;
		}

	void Set( float InX, float InY, float InZ )
		{
		x = InX;
		y = InY;
		z = InZ;
		}
	// Operator Overloads
	inline bool operator== (const CVec3& V2) const 
		{
		return (x == V2.x && y == V2.y && z == V2.z);
		}

	inline CVec3 operator+ (const CVec3& V2) const 
		{
		return CVec3( x + V2.x,  y + V2.y,  z + V2.z);
		}
	inline CVec3 operator- (const CVec3& V2) const
		{
		return CVec3( x - V2.x,  y - V2.y,  z - V2.z);
		}
	inline CVec3 operator- ( ) const
		{
		return CVec3(-x, -y, -z);
		}

	inline CVec3 operator/ (float S ) const
		{
		float fInv = 1.0f / S;
		return CVec3 (x * fInv , y * fInv, z * fInv);
		}
	inline CVec3 operator/ (const CVec3& V2) const
		{
		return CVec3 (x / V2.x,  y / V2.y,  z / V2.z);
		}
	inline CVec3 operator* (const CVec3& V2) const
		{
		return CVec3 (x * V2.x,  y * V2.y,  z * V2.z);
		}
	inline CVec3 operator* (float S) const
		{
		return CVec3 (x * S,  y * S,  z * S);
		}
	inline void operator += ( const CVec3& V2 ) 
		{
		x += V2.x;
		y += V2.y;
		z += V2.z;
		}

	// Functions
	inline float Dot( const CVec3 &V1 ) const
		{
		return V1.x*x + V1.y*y + V1.z*z;
		}

	inline CVec3 Cross( const CVec3 &V2 )  const
		{
		return CVec3(
			y * V2.z  -  z * V2.y,
			z * V2.x  -  x * V2.z,
			x * V2.y  -  y * V2.x 	);
		}

	// These require math.h for the sqrtf function
	float Magnitude( ) const
		{
		return sqrtf( x*x + y*y + z*z );
		}

	float Distance( const CVec3 &V1 ) const
		{
		return ( *this - V1 ).Magnitude();	
		}

	inline void Normalize()
		{
		float fMag = ( x*x + y*y + z*z );
		if (fMag == 0) {return;}

		float fMult = 1.0f/sqrtf(fMag);            
		x *= fMult;
		y *= fMult;
		z *= fMult;
		return;
		}
};

// 
// 2 Element Vector Bonus Class!
//
class CVec2
{
public:
	// Data
	float x, y;

	// Ctors
	CVec2( float InX, float InY ) : x(InX), y(InY)
		{
		}
	CVec2( ) : x(0), y(0)
		{
		}
	// Setting
	void Set( float InX, float InY )
		{
		x = InX;
		y = InY;
		}
	// Operator Overloads
	inline bool operator== (const CVec2& V2) const 
		{
		return (x == V2.x && y == V2.y);
		}
	inline CVec2 operator+ (const CVec2& V2) const 
		{
		return CVec2( x + V2.x,  y + V2.y );
		}
	inline CVec2 operator- (const CVec2& V2) const
		{
		return CVec2( x - V2.x,  y - V2.y );
		}
	inline CVec2 operator- ( ) const
		{
		return CVec2(-x, -y );
		}
	inline CVec2 operator/ (float S ) const
		{
		float fInv = 1.0f / S;
		return CVec2 (x * fInv , y * fInv );
		}
	inline CVec2 operator/ (const CVec2& V2) const
		{
		return CVec2 (x / V2.x,  y / V2.y );
		}
	inline CVec2 operator* (const CVec2& V2) const
		{
		return CVec2 (x * V2.x,  y * V2.y );
		}
	inline CVec2 operator* (float S) const
		{
		return CVec2 (x * S,  y * S );
		}
	inline void operator += ( const CVec2& V2 ) 
		{
		x += V2.x;
		y += V2.y;
		}
	inline void operator /= ( float R ) 
		{
		x /= R;
		y /= R;
		}
	float Magnitude( ) const
		{
		return sqrtf( x*x + y*y );
		}
};

// fixme: Move this some place more sensible
struct SColor 
{
	float r, g, b;

	void White() { r=1.0f; g=1.0f; b=1.0f; }
	void SetUChar( unsigned char inR, unsigned char inG, unsigned char inB )
	{
		r = inR * 255.0f;
		g = inG * 255.0f;
		b = inB * 255.0f;
	}
};

#endif